/* $Id: Hash_map_iterator.h,v 1.7 2003/03/12 20:16:22 lergnom Exp $ -*- C++ -*-
 * 
 * libbpfl - Bayesian Probability Filtering Library
 * Copyright (C) 2003 Anthony Ventimiglia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
#ifndef BPF__HASH_MAP_ITERATOR_H
#define BPF__HASH_MAP_ITERATOR_H

// Check to make sure this is included from bpfHash_map.h
#ifndef BPF_HASH_MAP_H
#error This header should never be used directly, it is used by bpfHash_map.h
#endif

// This is not to be included directly, it is included by bpfHash_map.h

#ifdef __BPF_BUILD
#include "common.h"
#endif 



///This is used both for sequential and random access Hash_map operations.
///This is really only to be used to separate the interface from the Hash_map
///interface, since all that template code is unsightly.
///
///It really cannot be used alone, since it is a child of a bpfHash_map.
template <typename KeyType, typename ValueType, typename HashObject>
class Hash_map<KeyType, ValueType, HashObject>::iterator
{
public:
    iterator ( Hash_map& m, const size_t &i ) 
	: map ( m ), ind ( i ) {}
    ///Copy constructor
    iterator ( const iterator& x ) : map ( x.map ), ind ( x.ind ) {}
    
    ///Prefix increment, finds next element in map.
    iterator operator++ () const	
	{
	    for ( ind++; ind < map.size() && !map.table[ind].in_use; ind++);
	    return *this;
	}
    iterator operator++ ( int ) const { return operator++(); }// Postfix

    ///Comparison operation, can be used for all boolean funcs
    bool compare ( const iterator& x ) const { return ind - x.ind; }
    bool operator== ( const iterator& x ) const { return compare( x ) == 0; }
    bool operator!= ( const iterator& x ) const { return compare( x ) != 0; }
    
    ///Sets cell value.
    ///Using an iterator from find() set the ValueType part of the cell.
    ///@todo Do this more like the STL iterators, and move it into the
    ///      Not yet designed Hash_map::iterator class.
    ///
    iterator operator= ( const ValueType& v )
	{
	    ///@todo add bounds checking here.
	    map.table[ind].value = v; 
	    return *this;
	}
    iterator operator+= ( const ValueType& v ) 
	{ return operator= ( map.table[ind].value + v ); }
    
    ///Return Key using iterator.
    KeyType key () const
	{
	    ///@todo check bounds
	    return map.table[ind].key;
	}
    
    ///Return Value using iterator.
    ValueType operator* () const 
	{
	    ///@todo check bounds
	    return map.table[ind].value;
	}

private:

    /// For my convenience
    typedef Hash_map<KeyType, ValueType, HashObject> map_class;
    typedef map_class::iterator self_type;
    Hash_map<KeyType, ValueType, HashObject>& map;
    mutable Hash_map_base::tablesize ind;
};

#endif // BPF__HASH_MAP_ITERATOR_H
