/* $Id: Hash-map-test-int.cpp,v 1.4 2003/03/10 20:10:37 lergnom Exp $ -*- C++ -*-
 * 
 * libbpfl - Bayesian Probability Filtering Library
 * Copyright (C) 2003 Anthony Ventimiglia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

/**
 * @file   Hash-test.cpp
 * @author Anthony Ventimiglia
 * @date   Fri Mar  7 13:08:06 2003
 * 
 * @brief  This is both a test of the bpfHash_map, and an example of it use.
 * In this example we'll make a Hash_map<int>
 * 
 * 
 */


#include "bpfHash_map.h"
#include "bpfHash.h"

#include <iostream>


/// This is an example of how to define the hash function. You have to make
/// sure this is linked to whatever modules instantiate the Hash_map<int>
/// template
struct Hash : public bpf::Hash_base
{
    size_t operator() ( const size_t& k ) { return k; }
};
    


int main()
{
    using namespace std;
    const int LSize = 90;
    int numList[] = { 123, 123, 123, 123, 123, 123, 123, 123, 123, 
		      234, 234, 234, 234, 234, 234, 234, 234, 345,
		      345, 345, 345, 345, 345, 345, 456, 456, 456,
		      456, 456, 456, 567, 567, 567, 567, 567, 678,
		      678, 678, 678, 789, 789, 789, 890, 890, 900,
		      987, 987, 987, 987, 987, 987, 987, 987, 987,
		      876, 876, 876, 876, 876, 876, 876, 876, 765, 
		      765, 765, 765, 765, 765, 765, 654, 654, 654, 
		      654, 654, 654, 543, 543, 543, 543, 543, 432, 
		      432, 432, 432, 321, 321, 321, 211, 211, 111 };
		      
			    
    // First we'll fill the Hash_map, and save the numbers put in in our
    // numList
    typedef bpf::Hash_map<int, int, Hash> IHash;
    IHash numTable;
    for ( int i = 0; i < LSize; ++i ) {
	IHash::iterator hi = numTable.find ( numList[i] );
	hi = *hi + 1;
    }

    // Now print numList and numTable
    cout << "Contents of numList:\n";
    for ( int i = 0; i < LSize; ++i )
	cout << i << ": " << numList[i] << endl;

    cout << "\nContents of numTable:\n";
    for ( IHash::iterator hi = numTable.begin(); hi != numTable.end(); ++hi )
	cout << hi.key () << " -> " << *hi << endl;

    return 0;
}
