/* $Id: Hash-map-test-string.cpp,v 1.3 2003/03/10 20:10:37 lergnom Exp $ -*- C++ -*-
 * 
 * libbpfl - Bayesian Probability Filtering Library
 * Copyright (C) 2003 Anthony Ventimiglia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
/**
 * @file   Hash-map-test-string.cpp
 * @author Anthony Ventimiglia
 * @date   Mon Mar 10 09:46:04 2003
 * 
 * @brief  This is a test and an example of the bpf::Hash_map use. It uses a
 * std::string as the KeyType, so is a little more complex than the previous
 * test
 */

#include "bpfHash_map.h"
#include "bpfHash.h"

#include <iostream>
#include <string>

const int LSize = 89;
char* wordList[] = { "abasement", "abasement", "abasement", "abasement", 
		     "abasement", "abasement", "abasement", "abasement", 
		     "abasement",  // 9 abasements
		     "beer", "beer", "beer", "beer", "beer", "beer", 
		     "beer", "beer", // 8 beers
		     "chancery", "chancery", "chancery", "chancery", 
		     "chancery", "chancery", "chancery", // 7 chanceries
		     "decently", "decently", "decently", "decently", 
		     "decently", "decently", // 6 decently
		     "emulator", "emulator", "emulator", "emulator", 
		     "emulator", // 5 emulators
		     "firewood", "firewood", "firewood", "firewood", // 4
		     "gram", "gram", "gram", // 3 grams
		     "heaver", "heaver", // 2 heavers
		     "infernally", // 1 infernally
		     "junta", "junta", // 2 juntas
		     "kinsman", "kinsman", "kinsman", // 3 kinsmen
		     "laboratories", "laboratories", "laboratories",
		     "laboratories", // 4 laboratories
		     "marijuana", "marijuana", "marijuana", "marijuana", 
		     "marijuana", // 5 marijuanas
		     "neutrinos", "neutrinos", "neutrinos", "neutrinos", 
		     "neutrinos", "neutrinos", // 6 neutrinos
		     "orchids", "orchids", "orchids", "orchids", 
		     "orchids", "orchids", "orchids", // 7 orchids
		     "patties", "patties", "patties", "patties", 
		     "patties", "patties", "patties", "patties", // 8 pats
		     "RISC", "RISC", "RISC", "RISC", "RISC", "RISC", 
		     "RISC", "RISC", "RISC" }; // 9 RISCS 


/// Here is an example of the Hash function object class which can be
/// used to work with the Hash_map
struct Hash : public bpf::Hash_base 
{
    size_t operator() ( const std::string& word ) 
	{
	    size_t hval = 0;
	    typedef std::string::const_iterator SI;
	    for ( SI i = word.begin(); i != word.end(); ++i ) 
		hval = (hval << 1) ^ *i;
	    return hval;
	}
};


int main ()
{
    using namespace std;
    
    // First fill the Hash_map.
    typedef bpf::Hash_map<std::string, int, Hash> strHash;
    strHash wordTable;
    for ( int i = 0; i < LSize; ++i ) {
	std::string str ( wordList[i] );
	strHash::iterator hi = wordTable.find ( str );
	hi += 1;
    }

    // Now print wordList and table
    cout << "Contents of wordList:\n";
    for ( int i = 0; i < LSize; ++i )
	cout << i << ": " << wordList[i] << endl;

    cout << "\nContents of wordTable:\n";
    for ( strHash::iterator hi = wordTable.begin(); hi != wordTable.end();
	  hi++ ) 
	cout << hi.key () << " -> " << *hi << endl;

    return 0;
}







